@@ -1,11 +1,11 @@
 ;;; cape.el --- Completion At Point Extensions -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021  Free Software Foundation, Inc.
+;; Copyright (C) 2021, 2022  Free Software Foundation, Inc.
 
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2021
-;; Version: 0.5
+;; Version: 0.7
 ;; Package-Requires: ((emacs "27.1"))
 ;; Homepage: https://github.com/minad/cape
 
@@ -60,12 +60,11 @@
   "Dictionary word list file."
   :type 'string)
 
-(defcustom cape-company-timeout 5.0
-  "Company asynchronous timeout."
-  :type '(choice nil float))
-
 (defcustom cape-dabbrev-min-length 4
-  "Minimum length of dabbrev expansions."
+  "Minimum length of dabbrev expansions.
+This setting ensures that words which are too short
+are not offered as completion candidates, such that
+auto completion does not pop up too aggressively."
   :type 'integer)
 
 (defcustom cape-dabbrev-check-other-buffers t
@@ -356,6 +355,15 @@
     (cape--silent
       (complete-with-action action table str pred))))
 
+(defun cape--nonessential-table (table)
+  "Mark completion TABLE as `non-essential'."
+  (lambda (str pred action)
+    (let ((non-essential t))
+      (let ((result (funcall table str pred action)))
+        (when (and (eq action 'completion--unquote) (functionp (cadr result)))
+          (cl-callf cape--nonessential-table (cadr result)))
+        result))))
+
 (cl-defun cape--table-with-properties (table &key category (sort t) &allow-other-keys)
   "Create completion TABLE with properties.
 CATEGORY is the optional completion category.
@@ -417,16 +425,19 @@ VALID is the input comparator, see `cape--input-valid-p'."
 ;;;###autoload
 (defun cape-file (&optional interactive)
   "Complete file name at point.
+See the user option `cape-file-directory-must-exist'.
 If INTERACTIVE is nil the function acts like a capf."
   (interactive (list t))
   (if interactive
-      (let (cape-file-directory-must-exist)
+      (let ((cape-file-directory-must-exist))
         (cape--interactive #'cape-file))
     (let* ((bounds (cape--bounds 'filename))
+           (non-essential t)
            (file (buffer-substring (car bounds) (cdr bounds))))
       (when (or (not cape-file-directory-must-exist)
                 (and (string-match-p "/" file) (file-exists-p (file-name-directory file))))
-        `(,(car bounds) ,(cdr bounds) ,#'read-file-name-internal
+        `(,(car bounds) ,(cdr bounds)
+          ,(cape--nonessential-table #'read-file-name-internal)
           ,@(and (not (equal file "/")) (string-suffix-p "/" file)
                  '(:company-prefix-length t))
           :exclusive no ,@cape--file-properties)))))
@@ -462,7 +473,7 @@ If INTERACTIVE is nil the function acts like a capf."
 
 ;;;###autoload
 (defun cape-symbol (&optional interactive)
-  "Complete symbol at point.
+  "Complete Elisp symbol at point.
 If INTERACTIVE is nil the function acts like a capf."
   (interactive (list t))
   (if interactive
@@ -490,6 +501,8 @@ If INTERACTIVE is nil the function acts like a capf."
 ;;;###autoload
 (defun cape-dabbrev (&optional interactive)
   "Complete with Dabbrev at point.
+See the user options `cape-dabbrev-min-length' and
+`cape-dabbrev-check-other-buffers'.
 If INTERACTIVE is nil the function acts like a capf."
   (interactive (list t))
   (if interactive
@@ -500,11 +513,12 @@ If INTERACTIVE is nil the function acts like a capf."
             (end (match-end 0)))
         `(,beg ,end
           ,(cape--table-with-properties
-            ;; Use equal, if candidates must be longer than cape-dabbrev-min-length.
             (cape--cached-table beg end
                                 #'cape--dabbrev-list
-                                (if (> cape-dabbrev-min-length 0)
-                                    'equal 'prefix))
+                                ;; TODO: Use equal, if candidates must be longer than cape-dabbrev-min-length.
+                                ;;(if (> cape-dabbrev-min-length 0) 'equal 'prefix)
+                                ;; Problem is that when entering more input, candidates get lost!
+                                'prefix)
             :category 'cape-dabbrev)
           :exclusive no ,@cape--dabbrev-properties)))))
 
@@ -535,7 +549,7 @@ If INTERACTIVE is nil the function acts like a capf."
 
 ;;;###autoload
 (defun cape-ispell (&optional interactive)
-  "Complete with Ispell at point.
+  "Complete word at point with Ispell.
 If INTERACTIVE is nil the function acts like a capf."
   (interactive (list t))
   (if interactive
@@ -560,13 +574,14 @@ If INTERACTIVE is nil the function acts like a capf."
   (or cape--dict-words
       (setq cape--dict-words
             (split-string (with-temp-buffer
-                            (insert-file-contents-literally cape-dict-file)
+                            (insert-file-contents cape-dict-file)
                             (buffer-string))
                           "\n" 'omit-nulls))))
 
 ;;;###autoload
 (defun cape-dict (&optional interactive)
-  "Complete word at point.
+  "Complete word from dictionary at point.
+See the custom option `cape-dict-file'.
 If INTERACTIVE is nil the function acts like a capf."
   (interactive (list t))
   (if interactive
@@ -582,9 +597,9 @@ If INTERACTIVE is nil the function acts like a capf."
 ;; macro for this computation since packages like `helpful' will
 ;; `macroexpand-all' the expensive `cape--define-char' macro calls.
 (eval-when-compile
-  (defun cape--char-translation (method prefix)
+  (defun cape--char-translation (method regexp)
     "Return character translation alist for METHOD.
-PREFIX is the prefix regular expression."
+REGEXP is the regular expression matching the names."
     (declare (pure t))
     (save-window-excursion
       (describe-input-method method)
@@ -599,7 +614,6 @@ PREFIX is the prefix regular expression."
                    "\\`\\(\n\\|.\\)*?KEY SEQUENCE\n-+\n" ""
                    (buffer-string))))
                 "\n"))
-              (regexp (concat "\\`" prefix))
               (list nil))
           (dolist (line lines)
             (let ((beg 0) (len (length line)))
@@ -615,11 +629,11 @@ PREFIX is the prefix regular expression."
           (kill-buffer)
           (sort list (lambda (x y) (string< (car x) (car y)))))))))
 
-(defmacro cape--char-define (name method prefix)
+(defmacro cape--char-define (name method &rest prefix)
   "Define character translation capf.
 NAME is the name of the capf.
 METHOD is the input method.
-PREFIX is the prefix regular expression."
+PREFIX are the prefix characters."
   (let ((capf (intern (format "cape-%s" name)))
         (prefix-required (intern (format "cape-%s-prefix-required" name)))
         (list (intern (format "cape--%s-list" name)))
@@ -628,7 +642,9 @@ PREFIX is the prefix regular expression."
         (exit (intern (format "cape--%s-exit" name)))
         (properties (intern (format "cape--%s-properties" name))))
     `(progn
-       (defvar ,list (cape--char-translation ,method ,prefix))
+       (defvar ,list (cape--char-translation
+                      ,method
+                      ,(concat "\\`" (regexp-opt (mapcar #'char-to-string prefix)))))
        (defcustom ,prefix-required t
          ,(format "Initial prefix is required for `%s' to trigger." capf)
          :type 'boolean)
@@ -662,10 +678,13 @@ is nil the function acts like a capf." method method)
          (if interactive
              ;; NOTE: Disable cycling since replacement breaks it.
              (let (completion-cycle-threshold ,prefix-required)
+               (when (memq last-input-event ',prefix)
+                 (self-insert-command 1 last-input-event))
                (cape--interactive #',capf))
            (when-let (bounds
                       (cond
-                       ((thing-at-point-looking-at ,(format "%s[^ \n\t]*" prefix))
+                       ((thing-at-point-looking-at
+                         ,(concat (regexp-opt (mapcar #'char-to-string prefix)) "[^ \n\t]*" ))
                         (cons (match-beginning 0) (match-end 0)))
                        ((not ,prefix-required) (cons (point) (point)))))
              (append
@@ -677,9 +696,9 @@ is nil the function acts like a capf." method method)
 ;;;###autoload (autoload 'cape-tex "cape" nil t)
 ;;;###autoload (autoload 'cape-sgml "cape" nil t)
 ;;;###autoload (autoload 'cape-rfc1345 "cape" nil t)
-(cape--char-define tex "TeX" "[\\\\^_]")
-(cape--char-define sgml "sgml" "&")
-(cape--char-define rfc1345 "rfc1345" "&")
+(cape--char-define tex "TeX" ?\\ ?^ ?_)
+(cape--char-define sgml "sgml" ?&)
+(cape--char-define rfc1345 "rfc1345" ?&)
 
 ;;;;; cape-abbrev
 
@@ -708,8 +727,8 @@ is nil the function acts like a capf." method method)
 
 (defun cape--abbrev-exit (_str status)
   "Expand expansion if STATUS is not exact."
-   (unless (eq status 'exact)
-     (expand-abbrev)))
+  (unless (eq status 'exact)
+    (expand-abbrev)))
 
 (defvar cape--abbrev-properties
   (list :annotation-function #'cape--abbrev-annotation
@@ -746,7 +765,8 @@ If INTERACTIVE is nil the function acts like a capf."
 
 ;;;###autoload
 (defun cape-keyword (&optional interactive)
-  "Complete word at point.
+  "Complete programming language keyword at point.
+See the variable `cape-keywords'.
 If INTERACTIVE is nil the function acts like a capf."
   (interactive (list t))
   (if interactive
@@ -801,13 +821,13 @@ If INTERACTIVE is nil the function acts like a capf."
     (when-let (results (delq nil (mapcar #'funcall capfs)))
       (pcase-let* ((`((,beg ,end . ,_)) results)
                    (cache-candidates nil)
-                   (cache-str nil)
+                   (cache-filter nil)
                    (cache-ht (make-hash-table :test #'equal))
                    (extra-fun
                     (lambda (prop)
-                      (lambda (x)
-                        (when-let (fun (plist-get (gethash x cache-ht) prop))
-                          (funcall fun x)))))
+                      (lambda (cand &rest args)
+                        (when-let (fun (plist-get (gethash cand cache-ht) prop))
+                          (apply fun cand args)))))
                    (tables nil)
                    (prefix-len nil))
         (cl-loop for (beg2 end2 . rest) in results do
@@ -831,25 +851,26 @@ If INTERACTIVE is nil the function acts like a capf."
                               (display-sort-function . identity)
                               (cycle-sort-function . identity)))
                   ('t
-                   (unless (equal str cache-str)
-                     (let ((ht (make-hash-table :test #'equal))
-                           (candidates nil))
-                       (cl-loop for (table . plist) in tables do
-                                (let* ((pr (plist-get plist :predicate))
-                                       (md (completion-metadata "" table pr))
-                                       (sort (or (completion-metadata-get md 'display-sort-function)
-                                                 #'identity))
-                                       (cands (funcall sort (all-completions str table pr))))
-                                  (cl-loop for cell on cands
-                                           for cand = (car cell) do
-                                           (if (and (eq (gethash cand ht t) t)
-                                                    (or (not pred) (funcall pred cand)))
-                                               (puthash cand plist ht)
-                                             (setcar cell nil)))
-                                  (setq candidates (nconc candidates cands))))
-                       (setq cache-str str
-                             cache-candidates (delq nil candidates)
-                             cache-ht ht)))
+                   (let ((filter (list str (copy-sequence completion-regexp-list) completion-ignore-case)))
+                     (unless (equal filter cache-filter)
+                       (let ((ht (make-hash-table :test #'equal))
+                             (candidates nil))
+                         (cl-loop for (table . plist) in tables do
+                                  (let* ((pr (plist-get plist :predicate))
+                                         (md (completion-metadata "" table pr))
+                                         (sort (or (completion-metadata-get md 'display-sort-function)
+                                                   #'identity))
+                                         (cands (funcall sort (all-completions str table pr))))
+                                    (cl-loop for cell on cands
+                                             for cand = (car cell) do
+                                             (if (and (eq (gethash cand ht t) t)
+                                                      (or (not pred) (funcall pred cand)))
+                                                 (puthash cand plist ht)
+                                               (setcar cell nil)))
+                                    (setq candidates (nconc candidates cands))))
+                         (setq cache-filter filter
+                               cache-candidates (delq nil candidates)
+                               cache-ht ht))))
                    (copy-sequence cache-candidates))
                   (_
                    (completion--some
@@ -864,7 +885,7 @@ If INTERACTIVE is nil the function acts like a capf."
               :company-deprecated (funcall extra-fun :company-deprecated)
               :company-kind (funcall extra-fun :company-kind)
               :annotation-function (funcall extra-fun :annotation-function)
-              :exit-function (lambda (x _status) (funcall (funcall extra-fun :exit-function) x)))))))
+              :exit-function (lambda (x s) (funcall (funcall extra-fun :exit-function) x s)))))))
 
 (defun cape--company-call (&rest app)
   "Apply APP and handle future return values."
@@ -874,26 +895,29 @@ If INTERACTIVE is nil the function acts like a capf."
     (pcase (apply app)
       ;; Handle async future return values.
       (`(:async . ,fetch)
-       (let ((res 'cape--waiting)
-             (start (time-to-seconds)))
-         (unwind-protect
-             (progn
-               (funcall fetch (lambda (arg)
-                                (when (eq res 'cape--waiting)
-                                  (push 'cape--done unread-command-events))
-                                (setq res arg)))
-               ;; Force synchronization.
-               (while (eq res 'cape--waiting)
-                 ;; When we've got input, interrupt the computation.
-                 (when (and unread-command-events toi)
-                   (throw toi nil))
-                 (when (and cape-company-timeout
-                            (> (- (time-to-seconds) start) cape-company-timeout))
-                   (error "Cape company backend async timeout"))
-                 (sit-for 0.1 'noredisplay)))
-           ;; Remove cape--done introduced by future callback
-           (setq unread-command-events
-                 (delq 'cape--done unread-command-events)))
+       (let ((res 'cape--waiting))
+         (if toi
+             (unwind-protect
+                 (progn
+                   (funcall fetch
+                            (lambda (arg)
+                              (when (eq res 'cape--waiting)
+                                (push 'cape--done unread-command-events)
+                                (setq res arg))))
+                   (when (eq res 'cape--waiting)
+                     (let ((ev (let (input-method-function) (read-event nil t))))
+                       (unless (eq ev 'cape--done)
+                         (push (cons t ev) unread-command-events)
+                         (setq res 'cape--cancelled)
+                         (throw toi t)))))
+               (setq unread-command-events
+                     (delq 'cape--done unread-command-events)))
+           (funcall fetch (lambda (arg) (setq res arg)))
+           ;; Force synchronization, not interruptible! We use polling
+           ;; here and ignore pending input since we don't use
+           ;; `sit-for'. This is the same method used by Company itself.
+           (while (eq res 'cape--waiting)
+             (sleep-for 0.01)))
          res))
       ;; Plain old synchronous return value.
       (res res))))
@@ -910,19 +934,23 @@ This feature is experimental."
         (set init t))
       (when-let* ((prefix (cape--company-call backend 'prefix))
                   (initial-input (if (stringp prefix) prefix (car-safe prefix))))
-        (let* ((end (point)) (beg (- end (length initial-input))))
+        (let* ((end (point)) (beg (- end (length initial-input)))
+               (dups (cape--company-call backend 'duplicates))
+               candidates)
           (list beg end
                 (funcall
                  (if (cape--company-call backend 'ignore-case)
                      #'completion-table-case-fold
                    #'identity)
                  (cape--table-with-properties
-                  (cape--cached-table beg end
-                                      (if (cape--company-call backend 'duplicates)
-                                          (lambda (input)
-                                            (delete-dups (cape--company-call backend 'candidates input)))
-                                        (apply-partially #'cape--company-call backend 'candidates))
-                                      (if (cape--company-call backend 'no-cache initial-input) 'never valid))
+                  (cape--cached-table
+                   beg end
+                   (lambda (input)
+                     (setq candidates (cape--company-call backend 'candidates input))
+                     (when dups (setq candidates (delete-dups candidates)))
+                     candidates)
+                   (if (cape--company-call backend 'no-cache initial-input)
+                       'never valid))
                   :category backend
                   :sort (not (cape--company-call backend 'sorted))))
                 :exclusive 'no
@@ -933,13 +961,23 @@ This feature is experimental."
                 :company-deprecated (lambda (x) (cape--company-call backend 'deprecated x))
                 :company-kind (lambda (x) (cape--company-call backend 'kind x))
                 :annotation-function (lambda (x) (cape--company-call backend 'annotation x))
-                :exit-function (lambda (x _status) (cape--company-call backend 'post-completion x))))))))
+                :exit-function
+                (lambda (x _status)
+                  (cape--company-call backend 'post-completion
+                                      (or (car (member x candidates)) x)))))))))
 
 ;;;###autoload
-(defun cape-capf-buster (capf &optional valid)
-  "Return transformed CAPF where the cache is busted on input change.
-VALID is the input comparator, see `cape--input-valid-p'."
-  (lambda ()
+(defun cape-interactive-capf (capf)
+  "Create interactive completion function from CAPF."
+  (lambda (&optional interactive)
+    (interactive (list t))
+    (if interactive (cape--interactive capf) (funcall capf))))
+
+;;;###autoload
+(defun cape-wrap-buster (capf &optional valid)
+  "Call CAPF and return a completion table with cache busting.
+The cache is busted when the input changes, where VALID is the input
+comparator, see `cape--input-valid-p'."
     (pcase (funcall capf)
       (`(,beg ,end ,table . ,plist)
        `(,beg ,end
@@ -955,76 +993,100 @@ VALID is the input comparator, see `cape--input-valid-p'."
                           ;; An interruption should not happen between the setqs.
                           (setq table new-table input new-input)))))
                    (complete-with-action action table str pred)))
-              ,@plist)))))
+              ,@plist))))
 
 ;;;###autoload
-(defun cape-capf-with-properties (capf &rest properties)
-  "Return a new CAPF with additional completion PROPERTIES.
+(defun cape-wrap-properties (capf &rest properties)
+  "Call CAPF and add additional completion PROPERTIES.
 Completion properties include for example :exclusive, :annotation-function and
 the various :company-* extensions. Furthermore a boolean :sort flag and a
 completion :category symbol can be specified."
-  (lambda ()
-    (pcase (funcall capf)
-      (`(,beg ,end ,table . ,plist)
-       `(,beg ,end
-              ,(apply #'cape--table-with-properties table properties)
-              ,@properties ,@plist)))))
+  (pcase (funcall capf)
+    (`(,beg ,end ,table . ,plist)
+     `(,beg ,end
+            ,(apply #'cape--table-with-properties table properties)
+            ,@properties ,@plist))))
 
 ;;;###autoload
-(defun cape-capf-with-predicate (capf predicate)
-  "Return a new CAPF with an additional candidate PREDICATE.
+(defun cape-wrap-predicate (capf predicate)
+  "Call CAPF and add an additional candidate PREDICATE.
 The PREDICATE is passed the candidate symbol or string."
-  (lambda ()
-    (pcase (funcall capf)
-      (`(,beg ,end ,table . ,plist)
-       `(,beg ,end ,table
-              :predicate
-              ,(if-let (pred (plist-get plist :predicate))
-                   ;; First argument is key, second is value for hash tables.
-                   ;; The first argument can be a cons cell for alists. Then
-                   ;; the candidate itself is either a string or a symbol. We
-                   ;; normalize the calling convention here such that PREDICATE
-                   ;; always receives a string or a symbol.
-                   (lambda (&rest args)
-                     (when (apply pred args)
-                       (setq args (car args))
-                       (funcall predicate (if (consp args) (car args) args))))
-                 (lambda (key &optional _val)
-                   (funcall predicate (if (consp key) (car key) key))))
-              ,@plist)))))
+  (pcase (funcall capf)
+    (`(,beg ,end ,table . ,plist)
+     `(,beg ,end ,table
+            :predicate
+            ,(if-let (pred (plist-get plist :predicate))
+                 ;; First argument is key, second is value for hash tables.
+                 ;; The first argument can be a cons cell for alists. Then
+                 ;; the candidate itself is either a string or a symbol. We
+                 ;; normalize the calling convention here such that PREDICATE
+                 ;; always receives a string or a symbol.
+                 (lambda (&rest args)
+                   (when (apply pred args)
+                     (setq args (car args))
+                     (funcall predicate (if (consp args) (car args) args))))
+               (lambda (key &optional _val)
+                 (funcall predicate (if (consp key) (car key) key))))
+            ,@plist))))
 
 ;;;###autoload
-(defun cape-silent-capf (capf)
-  "Create a new CAPF which is silent (no messages, no errors)."
-  (lambda ()
-    (pcase (cape--silent (funcall capf))
-      (`(,beg ,end ,table . ,plist)
-       `(,beg ,end ,(cape--silent-table table) ,@plist)))))
+(defun cape-wrap-silent (capf)
+  "Call CAPF and silence it (no messages, no errors)."
+  (pcase (cape--silent (funcall capf))
+    (`(,beg ,end ,table . ,plist)
+     `(,beg ,end ,(cape--silent-table table) ,@plist))))
 
 ;;;###autoload
-(defun cape-capf-case-fold (capf &optional dont-fold)
-  "Create a new CAPF which is case insensitive.
-If DONT-FOLD is non-nil, return a completion table that is
-case sensitive instead."
-  (lambda ()
-    (pcase (funcall capf)
-      (`(,beg ,end ,table . ,plist)
-       `(,beg ,end ,(completion-table-case-fold table dont-fold) ,@plist)))))
+(defun cape-wrap-case-fold (capf &optional dont-fold)
+  "Call CAPF and return a case insenstive completion table.
+If DONT-FOLD is non-nil return a case sensitive table instead."
+  (pcase (funcall capf)
+    (`(,beg ,end ,table . ,plist)
+     `(,beg ,end ,(completion-table-case-fold table dont-fold) ,@plist))))
 
 ;;;###autoload
-(defun cape-noninterruptible-capf (capf)
-  "Create a new CAPF which is non-interruptible silent by input."
-  (lambda ()
-    (pcase (funcall capf)
-      (`(,beg ,end ,table . ,plist)
-       `(,beg ,end ,(cape--noninterruptible-table table) ,@plist)))))
+(defun cape-wrap-noninterruptible (capf)
+  "Call CAPF and return a non-interruptible completion table."
+  (pcase (funcall capf)
+    (`(,beg ,end ,table . ,plist)
+     `(,beg ,end ,(cape--noninterruptible-table table) ,@plist))))
 
 ;;;###autoload
-(defun cape-interactive-capf (capf)
-  "Create interactive completion function from CAPF."
-  (lambda (&optional interactive)
-    (interactive (list t))
-    (if interactive (cape--interactive capf) (funcall capf))))
+(defun cape-wrap-purify (capf)
+  "Call CAPF and ensure that it does not modify the buffer."
+  ;; bug#50470: Fix Capfs which illegally modify the buffer or which
+  ;; illegally call `completion-in-region'. The workaround here has been
+  ;; proposed @jakanakaevangeli in bug#50470 and is used in
+  ;; @jakanakaevangeli's capf-autosuggest package.
+  (catch 'cape--illegal-completion-in-region
+    (condition-case nil
+        (let ((buffer-read-only t)
+              (inhibit-read-only nil)
+              (completion-in-region-function
+               (lambda (beg end coll pred)
+                 (throw 'cape--illegal-completion-in-region
+                        (list beg end coll :predicate pred)))))
+          (funcall capf))
+      (buffer-read-only nil))))
+
+(defmacro cape--capf-wrapper (wrapper)
+  "Create a capf transformer for WRAPPER."
+  `(defun ,(intern (format "cape-capf-%s" wrapper)) (&rest args)
+     (lambda () (apply #',(intern (format "cape-wrap-%s" wrapper)) args))))
+
+;;;###autoload (autoload 'cape-capf-noninterruptible "cape")
+;;;###autoload (autoload 'cape-capf-case-fold "cape")
+;;;###autoload (autoload 'cape-capf-silent "cape")
+;;;###autoload (autoload 'cape-capf-predicate "cape")
+;;;###autoload (autoload 'cape-capf-properties "cape")
+;;;###autoload (autoload 'cape-capf-buster "cape")
+(cape--capf-wrapper noninterruptible)
+(cape--capf-wrapper case-fold)
+(cape--capf-wrapper silent)
+(cape--capf-wrapper predicate)
+(cape--capf-wrapper properties)
+(cape--capf-wrapper buster)
+(cape--capf-wrapper purify)
 
 (provide 'cape)
 ;;; cape.el ends here
